# Testing for the utility functions of the package

if(FALSE) {
	library( "RUnit" )
	library( "xmapcore" )
}

###############################################################################
## Create a test environment
##
.test.env = new.env( hash=TRUE, parent=emptyenv() )

###############################################################################
## SetUp is called before each test method
##
.setUp = function() {
}

###############################################################################
## TearDown is called after each test method
##
.tearDown = function() {
}

test.needs.array = function() {
	checkTrue( !xmapcore:::.needs.array( 'all', 'gene' ),           "needs array for 'all' actions should be false" )
	checkTrue( !xmapcore:::.needs.array( 'all', 'transcript' ),     "needs array for 'all' actions should be false" )
	checkTrue( !xmapcore:::.needs.array( 'all', 'exon' ),           "needs array for 'all' actions should be false" )

	checkTrue( !xmapcore:::.needs.array( 'details', 'gene' ),       "needs array for gene.details should be false" )
	checkTrue( !xmapcore:::.needs.array( 'details', 'transcript' ), "needs array for transcript.details should be false" )
	checkTrue( !xmapcore:::.needs.array( 'details', 'exon' ),       "needs array for exon.details should be false" )

	# This should be true, as despite it being an array, the array is not needed as a silent secondary parameter
	# The array in this case is the primary parameter (ids)
	checkTrue( !xmapcore:::.needs.array( 'details', 'array' ),      "needs array for array.details should be false" )

	checkTrue( xmapcore:::.needs.array( 'details', 'probeset' ),      "anything with src probeset should need an array" )
	checkTrue( xmapcore:::.needs.array( 'details', 'probe' ),         "anything with src probe should need an array" )

	checkTrue( xmapcore:::.needs.array( 'to', 'gene', 'probeset' ),            "mapping to probeset destination should require an array" )
	checkTrue( xmapcore:::.needs.array( 'to', 'transcript', 'cdnaprobeset' ),  "mapping to cdnaprobeset destination should require an array" )
	checkTrue( xmapcore:::.needs.array( 'to', 'gene', 'probe' ),               "mapping to probe destination should require an array" )
	checkTrue( xmapcore:::.needs.array( 'to', 'probe', 'array' ),              "mapping to array destination should require an array" )
	checkTrue( xmapcore:::.needs.array( 'to', 'gene', 'exon_probeset' ),       "mapping to exon_probeset destination should require an array" )
}

test.make.params = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		# Check the quote escaping is working...
		ids = c( "a", "b", "c'", "hah';" )
		checkEquals( xmapcore:::.make.params( ids ), "a,b,c\\',hah\\';" )

		# Check the parameter splitting is working
		oldmax = xmap.get.param( "max.query" )
		xmap.set.param( max.query=20 )

		# Check with a list of short ids
		checkEquals( length( xmapcore:::.make.params( letters ) ), 3 )
		checkEquals( xmapcore:::.make.params( letters )[[1]], "a,b,c,d,e,f,g,h,i,j" )

		# Then check with ids that are bigger than the limit that we at least get one through per line
		values = c( "This is a long id (28 chars)", "Followed by another (30 chars)" )
		split = xmapcore:::.make.params( values )
		checkEquals( length( split ), 2, "with max.query set to 20, we should get each id on a new row" )
		checkEquals( split[[2]], values[2], "with max.query set to 20, we should get each id on a new row" )

		xmap.set.param( max.query=oldmax )

		# Get all the genes
		genes = all.genes()

		# Pick 5000 of them, and check the splitting works
		split = xmapcore:::.make.params( sample( genes, 5000 ) )
		checkEquals( 8, length( split ), 'Should be 8 elements in the list after the split' )
		checkTrue( is.list( split ), 'Should be a list when back from .make.params with multiple items' )
		checkTrue( all( unlist( lapply( split, function( vv ) is.character( vv ) ) ) ), 'Should be a list of character strings when back from .make.params with multiple items' )

		# Try with 10 genes
		split = xmapcore:::.make.params( sample( genes, 10 ) )
		checkEquals( 1, length( split ), 'Should be 1 element in the list after the split' )
		checkTrue( is.character( split ), 'Should be a character string when back from .make.params with one item' )
		xmap.disconnect()
	}
}

test.set.get.params = function() {
	# check single assignment
	xmap.set.param( testing=TRUE )
	checkTrue( xmap.get.param( "testing" ), "Testing should be equal to TRUE" )
  
	# And multiple assignment
	xmap.set.param( testing=FALSE, multiple="tim" )
	checkTrue( !xmap.get.param( "testing" ), "Testing should now be equal to FALSE" )
	checkEquals( xmap.get.param( "multiple" ), "tim" )  
  
	# And list parameters
	xmap.set.param( testing=c( 1, 2, 3 ) )
	checkEquals( length( xmap.get.param( "testing" ) ), 3, "Testing should have 3 elements" )

	xmap.set.param( testing=list( a=1, b=2, c=3 ) )
	checkEquals( xmap.get.param( "testing" )$b, 2, "Element 'b' should be 2" )

	# And functional parameters
	xmap.set.param( hi=function( a ) { paste( "Hi", a ) }, bye=function( a ) { paste( "Bye", a ) } )
	checkEquals( xmap.get.param( "hi" )( "Tim" ), "Hi Tim", "Testing should be a function returning a string" )
	checkEquals( xmap.get.param( "bye" )( "Tim" ), "Bye Tim", "Testing should be a function returning a string" )

	# A vector of functional parameters as a value
	fncs = c( function() { "function 1" }, function() { "function 2" } )
	xmap.set.param( fns=fncs )
	checkEquals( xmap.get.param( "fns" )[[1]](), "function 1", "Testing should be a function returning a string" )
	checkEquals( xmap.get.param( "fns" )[[2]](), "function 2", "Testing should be a function returning a string" )

	# And then a list of them
	fncs = list( a=function() { "function 1" }, b=function() { "function 2" } )
	xmap.set.param( fns=fncs )
	checkEquals( xmap.get.param( "fns" )[[1]](), "function 1", "Testing should be a function returning a string" )
	checkEquals( xmap.get.param( "fns" )[[2]](), "function 2", "Testing should be a function returning a string" )

	checkEquals( xmap.get.param( "fns" )$a(), "function 1", "Testing should be a function returning a string" )
	checkEquals( xmap.get.param( "fns" )$b(), "function 2", "Testing should be a function returning a string" )
}

test.array.type = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		# Check that we have selected a default array type (when we know that xmap.array.type is working)
		arr = xmapcore:::.xmap.internals$array
		checkTrue( !is.null( arr ), "We should have a default array type selected after xmap.connect" )
		
		# Then check that calling it with a stupid name fails
		checkException( xmap.array.type( 'InvalidId' ), "Setting an invalid array type should fail", silent=TRUE )
		
		# And check we still have the same array type set after this failure
		checkEquals( arr, xmapcore:::.xmap.internals$array, "Same array should be set after failed call to xmap.set.array" )
		
		xmap.disconnect()
	}
}

test.rangeapply = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		# Need to add more tests for the other range queries
		checkTrue( is.vector( xmap.range.apply( symbol.to.gene( c( 'tp53', 'lama3' ), as.vector=F ), exons.in.range, as.vector=T ) ), "Should return a vector for exons in range of 2 genes" )
		checkTrue( length(    xmap.range.apply( symbol.to.gene( c( 'tp53', 'lama3' ), as.vector=F ), exons.in.range, as.vector=T ) ) > 0, "Should return a some results for exons in range of 2 genes" )
		checkTrue( is(        xmap.range.apply( symbol.to.gene( c( 'tp53', 'lama3' ), as.vector=F ), exons.in.range, as.vector=F ), "RangedData" ), "Should return a RangedData object for exons in range of 2 genes" )
		checkTrue( dim(       xmap.range.apply( symbol.to.gene( c( 'tp53', 'lama3' ), as.vector=F ), exons.in.range, as.vector=F ) )[1] > 2, "Should return some RangedData rows for exons in range of 2 genes" )
	}
}

###############################################################################
## Stupid test to make sure its working...
##
test.reality <- function() {
	checkEqualsNumeric( 1, 1, "1 != 1?  PANIC!" )
}


